
/*
                                  SunSeeker
                                  by: Graig Pearen
                                  Prince George, BC, Canada
                                  graig.pearen@gmail.com
                                  
      Software Version: see "#define ver" below. The format is  yyyymmdd

      ++++ indicates notes from the programmer to himself for later action +++
      
      The objective for this contest was to replicate the functionality of the 
      assembly language software of the previous prototype using C++ and mpide.
      
      Several future enhancements have been added in comment blocks or have been
      started to show the intent. These will be completed after the contest deadline.
      For example, the previous prototype ran a software clock but not an RTCC so that
      will be added later along with internet, web server, and other software. Also,
      full command line parsing was not done previously but will be added here.
      
      This code is open source according to the rules of the
      2012 DesignSpark chipKIT Challenge MAX32 contest.

      Function names begin with capital letters. Variable names do not.
      
      Processor: PIC32MX795F512L

*/

#define  ver  0x20120325        // Software Version (revision date)

#define debug true              // enable some debugging code

#include "sunseeker.h"          // pin definitions and setup
//#include <Client.h>           // Ethernet stuff
//#include <Ethernet.h>
//#include <Server.h>
//#include <Udp.h>

//#include <SPI.h>              // I2C (EEPROM)
//#include <OneWire.h>          // 1-Wire drivers for DS18S20 temperature sensors
#include <rtcc.h>               // real time clock calender in the PIC
//#include <alarm_interrupts.c>

/*                                        <><><><><><><><><><><><><><>
                                          <>                        <>
                                          <>   GLOBAL VARIABLES     <>
                                          <>                        <>
                                          <><><><><><><><><><><><><><>
*/

byte        lightEast = 0;          // east light level
byte        lightWest = 0;          // west light level
byte        lightUp   = 0;          // up light level
byte        lightDown = 0;          // down light level
byte        lightSun  = 0;          // sun light level
byte        lightSnow = 0;          // snow light level
int         azp       = 0;          // azimuth pulse count (max= 32,768)
int         elp       = 0;          // elevation pulse count (max= 32,768)
int         calPulse  = 0;          // temporary storage for calibrating the PPH
byte        pph       = 0;          // pulses per hour for cloudy day tracking

int         elpStow   = 0;          // saved elevation count when the array is stowed
int         azi       = 0;          // azimuth motor current in ma
int         eli       = 0;          // elevation motor current in ma
byte        azPark    = AZPARK;     // azimuth count for parking (user settable)
byte        elPark    = ELPARK;     // elevation count for parking (user settable)
byte        elStow    = ELSTOW;     // elevation count for stowing the array (user settable)
byte        skyCondition = DARK;    // sky condition (dark, cloudy, hazy, sunny, dusk, dawn)
  
byte        pvTemp    = 0xff;       // PV temperature (FF if not set)
byte        airTemp   = 0xff;       // air temperature (FF if not set)
byte        airMin    = 0xff;       // today's minimum air temperature
word        minTime   = 0xffff;     // time that today's minimum temperature was recorded
byte        airMax    = 0xff;       // today's maximum air temperature
word        maxTime   = 0xffff;     // time that today's maximum temperature was recorded
int         dataSun   = 0;          // daily total sunny time in seconds
int         dataHaze  = 0;          // daily total hazy time in seconds
int         dataCloud = 0;          // daily total cloudy time in seconds
  
byte        year      = 0x12;       // default = 2012
byte        month     = 0x03;       // default = March
byte        mday      = 0x27;       // default = 27th
byte        wday      = 0x02;       // default = Tuesday
byte        hour      = 0x13;       // default = 1PM
byte        minute    = 0x21;       // default = 21 minutes
byte        second    = 0;

byte        brakeTimer= 0;          // motor brakes are on (1 second increments)
byte        minuteTimer = 0;        // delay minutes (used by park & calibrate)

boolean     manual    = false;      // manual mode flag
boolean     vertical  = false;      // array in vertical position (any of several reasons)
boolean     snowDump  = false;      // snow dump - vibrate every 10 minutes (automatic or manual)
boolean     snowMode  = false;      // tracking with the array vertical
boolean     singleAxis= false;      // single axis tracking mode
boolean     oneWay    = false;      // uni-directional east to west tracking only
boolean     coldStart = true;       // print sign on message
byte        rxChar    = 0;          // character received from computer (command to execute)
byte        suspend   = 0;          // suspended task (values are zero, P, X, Y)

byte        vector    = 0;          // array direction of movement (values are zero,E,W,U,D)
byte        mode      = 0;          // operating mode (values are zero,P,T,X)park, track,stow,snow)
long        pulseWidth= 0;          // motor pulse width in microseconds

//    --- FAULT FLAGS FOR MESSAGES & DIAGNOSTICS ---

boolean    azfail       = false;     // azimuth trouble...can not track azimuth
boolean    azCurrentHi  = false;     // motor overloaded
boolean    azCurrentLow = false;     // no load on the motor
boolean    azNoPulse    = false;     // array not moving
boolean    azHbridge    = false;     // H-bridge fault flag
boolean    azLimitSw    = false;     // auxiliary limit switch
boolean    azWest       = false;     // moving west

boolean    elfail       = false;     // elevation fail...can not track elevation
boolean    elCurrentHi  = false;     // motor overloaded
boolean    elCurrentLow = false;     // no load on the motor
boolean    elNoPulse    = false;     // array not moving
boolean    elHbridge    = false;     // H-bridge fault flag
boolean    elLimitSw    = false;     // auxiliary limit switch
boolean    elUp         = false;     // moving up

byte        fault     = 0;           // hardware fault code (cleared by software)
byte        alarm     = 0;           // alarm code (same as fault code but manually reset)
byte        warning   = 0;           // warning messages
byte        bug       = 0;           // software error code

boolean     verbose   = false;       // verbose is a test mode where SunSeeker talks a lot
boolean     doDay     = false;       // do daily jobs (midnight)
boolean     doHour    = false;       // do hourly tasks
boolean     doMinute  = false;       // do minute tasks
boolean     doSecond  = false;       // do second tasks
boolean     doCal     = false;       // counting pulses for pph calibration
   
byte        mac[]     = {0x00,0x18,0x3E,0x00,0xF5,0x15};    // the media access control
byte        ip[]      = {192,168,100,177};                  // default IP address
byte        gateway[] = {192,168,100,253};                  // default gateway
byte        subnet[]  = {255,255,255,0};                    // default subnet

unsigned long oldMillis = 0;                                // +++ not needed after RTCC is enabled +++
 
/*                                        <><><><><><><><><><><><><><>
                                          <>                        <>
                                          <>  PROTOTYPE FUNCTIONS   <>
                                          <>                        <>
                                          <><><><><><><><><><><><><><>
*/
void Track();                     // track the sun, returns with a fault code (0 = ok)
void Light();                     // read all the light sensors and save the results
void Halt();                      // stop all the morors (stop is reserved word)
void BrakeOff();                  // disable the motor brakes
void East();                      // turn the array east
void West();                      // turn the array west 
void Up();                        // move the array up
void Down();                      // move the array down
byte WatchMotor();                // motor control
byte Park();                      // park the array facing tomorrow's sunrise
byte Stow();                      // stow the array flat during high winds
byte UnStow();                    // restore the array position after a stow command
byte GoVertical();                // move the array to the vertical position
byte Scan();                      // scan west looking for the sun

byte Temperature(byte);           // read the air & PV temperatures
void Command(char);               // command line interpreter

void PrintSeconds (int);          // convert seconds to hh:mm and print it
byte PrintTemp();                 // format a temperature for printing (signed byte)
void PrintTime(byte, byte, byte); // print the time (hh:mm:ss)

void MotorTrouble();              // analyze a motor alarm & attempt a recovery
void ProcessWarning();            // report any warning (status) messages
void ProcessBug();                // process a software error trap
void MessageHeader(byte);         // print the message header (message number, date, time)
void CmdQuery();                  // Query command - report all data

/*                                        <><><><><><><><><><><><><>
                                          <>                      <>
                                          <>         SETUP        <>
                                          <>                      <>
                                          <><><><><><><><><><><><><>

    PTEMP, ATEMP, SDA1, SDA2 data direction is switched between input and OUTPUT by the driver software
*/

void setup() {
  
  pinMode (pinUpSensor,INPUT);        // up light sensor
  pinMode (pinDownSensor,INPUT);      // down light sensor
  pinMode (pinEastSensor,INPUT);      // east light sensor
  pinMode (pinWestSensor,INPUT);      // west light sensor
  pinMode (pinSunSensor,INPUT);       // sun light sensor
  pinMode (pinSnowSensor,INPUT);      // snow (reflected light) 
  pinMode (pinAzFeedback,INPUT);      // azimuth motor current
  pinMode (pinElFeedback,INPUT);      // elevation motor current

  pinMode (pinSCL1,OUTPUT);           // EEPROM clock
  pinMode (pinSCL2,OUTPUT);           // sensor module clock
  pinMode (pinVDD,OUTPUT);            // 3.3v power to the sensor module

  pinMode (pinHBridgeFault,INPUT);    // H-bridge fault sense input
  pinMode (pinElEnable,OUTPUT);       // H-bridge elevation enable
  pinMode (pinAzEnable,OUTPUT);       // H-bridge azimuth enable
  pinMode (pinHBridgeIn2,OUTPUT);     // H-bridge direction control input-2
  pinMode (pinHBridgeIn1,OUTPUT);     // H-bridge direction control input-1

  pinMode (pinPsDis,OUTPUT);          // 48-volt power supply disable
  pinMode (pinAP,INPUT);              // azimuth pulse input
  pinMode (pinEP,INPUT);              // elevation pulse input
  pinMode (pinAls,INPUT);             // optional azimuth limit switch input
  pinMode (pinEls,INPUT);             // optional elevation limit switch input
  pinMode (pinVib,OUTPUT);            // vibrator motor OUTPUT

  Serial.begin(9600);
//  Ethernet.begin(mac,ip,gateway);
//  server.begin();                     // start the internet server
  
/*
  RtccInit();			             // initialize the RTCC
  while(RtccGetClkStat()!=RTCC.CLK.ON);      // wait for the SOSC to start so RTCC has its clock source
  RtccSetTimeDate(0x10301500, 0x12032702);   // set the default time & date (2012 03 27 Tue, 10:30:15
*/

//  ------ INITIALIZE THE OUTPUT PINS ------

  digitalWrite (pinSCL1,LOW);           // EEPROM clock
  digitalWrite (pinSCL2,LOW);           // sensor module clock
  digitalWrite (pinVDD,LOW);            // 3.3v power to the sensor module

  digitalWrite (pinElEnable,LOW);       // H-bridge elevation enable
  digitalWrite (pinAzEnable,LOW);       // H-bridge azimuth enable
  digitalWrite (pinHBridgeIn2,LOW);     // H-bridge direction control input-2
  digitalWrite (pinHBridgeIn1,LOW);     // H-bridge direction control input-1

  digitalWrite (pinPsDis,HIGH);         // 48-volt power supply disable
  digitalWrite (pinVib,LOW);            // vibrator motor OUTPUT

  oldMillis = millis();                 // +++ not needed after RTCC is enabled +++


//             +++ SET UP SOME TEST DATA FOR CONTEST JUDGES +++

  if (debug){
    pvTemp    = 42;         // PV temperature (FF if not set)
    airTemp   = 28;         // air temperature (FF if not set)
    airMin    = 10;         // today's minimum air temperature
    minTime   = 0x0316;     // time that today's minimum temperature was recorded
    airMax    = 32;         // today's maximum air temperature
    maxTime   = 0x1245;     // time that today's maximum temperature was recorded
    dataSun   = 15300;      // daily total sunny time in seconds
    dataHaze  = 8250;       // daily total hazy time in seconds
    dataCloud = 1863;       // daily total cloudy time in seconds
  }
}                    // end of void setup()

//==========================================================================================================

/*                                        <><><><><><><><><><><><><>
                                          <>                      <>
                                          <>     MAIN PROGRAM     <>
                                          <>                      <>
                                          <><><><><><><><><><><><><>
*/

void loop() {

  if (coldStart){
      Serial.print("SunSeeker (c) 2012 G.Pearen Rev: ");  // print sign on message
      Serial.println(ver,HEX);                            // print the date without any separators (example 20120318)
      coldStart = false;
  }
  
  /*     +++ This clocking will run the software clock +++
             The RTCC sample code will not compile so I can't use its 1 sec interrupt
             This code will be removed when the RTCC is generating 1-second interrupts
  */
  if (oldMillis + 1000 <= millis()){
    oldMillis = millis();
    doSecond = true;
  }
  
  
// void Ethernet.PeriodicTasks()                    // keep the ethernet stack active

// ------ WATCH FOR A COMMAND TO PROCESS ------

if (Serial.available() > 0) {                     // anything from the computer?
  rxChar = Serial.read();                         // yes..get the character
  if (rxChar != 13){                              // ignore carriage returns for now
    Serial.println(rxChar);                       // echo it
    Command(rxChar);                              // process it
  }
}
  
// ------ WATCH THE MOTORS ------

  if (vector != 0) fault = WatchMotor();           // when a motor is running, watch for fault conditions
  if (fault > 0) MotorTrouble();                   // if there was a fault condition, deal with it

/*--------------------------------------- AUTOMATIC MODE ONLY ---------------------------------------------

                        These jobs are not run when SunSeeker is in manual mode 
      
        1- Enable tracking if it is bright enough
        2- Check pairs of light sensors and start tracking if required
        3- Stop the tracking when a pair of light sensors is balanced
*/

  if (! manual){                                 // don't track in manual mode (stowed sets manual mode)
  
    switch (skyCondition){
      case DARK:                                 // dark - can't track
        break;
        
      case DAWN:
        break;
        
      case DUSK:
        break;
        
      case CLOUDY:                               // cloudy day mode
        break;
      
      case HAZY:                                 // track the sun                        
        Track();
        break;
        
      case SUNNY:                                // track the sun
        Track();
        break;
        
      default:
        bug = BUG2;                               // software error trap
    } 

// ------ CHECK FOR SNOW ON THE ARRAY ------

    if(lightSun > DAWN){                          // is it bright enough to check for snow?
      if (! vertical){                            // if the array is not vertical, test for snow on the array
        if (lightSnow << 1 >= lightSun){          // light from snow > half the incident light
          snowDump = true;                        // set snow dump mode
          fault = GoVertical();                   // move the array to the vertical position
        }
      }
    }
// ------ STOP THE MOTOR WHEN LIGHT IS BALANCED ------

    if (lightEast == lightWest) {
      if (digitalRead(pinAzEnable)) Halt();      // stop tracking
    }
  
    if (lightUp == lightDown) {
      if (digitalRead(pinElEnable)) Halt();      // stop tracking
    }
  }                                              // end of if(! manual)

/* ------ ALWAYS DO THESE JOBS ------

          These jobs take a considerable amount of time to complete. When
          they are waiting, they suspend themselves and resume execution
          on the next pass through this routine. Only one job can be suspended
          at a time but since only 1 of these jobs can be running, that is ok.
*/

  if (suspend != 0){
    switch (suspend){                            // resume a suspended task
      case 'P':
        suspend = Park();                        // parking in progress
      case 'X':
        suspend = Stow();                        // stowing in progress
      case 'Y':
        suspend = UnStow();                      // unstowing in progress
      case 'S':
        suspend = Scan();
      default:
        bug = BUG3;                              // software error trap
    }
  }

/*================================ TIMED TASKS ========================================

    The time of day clock sets flags to initiate tasks tha are run on a regular
    time schedule. When the tasks are done, the flag is turned off and these tasks
    are then ignored until it is time to do them again.
    The falgs are doSecond, doMinute, do,Hour, and doDay. The daily tasks are run at midnight.
    
    +++ This code will have to be modified when the RTCC is operational +++
*/

/*-------------------------------- SECOND TASKS ----------------------------------

      1- Software clock
      2- Read the light levels
      3- Sunny, hazy, cloudy data update
      4- in verbose mode, print data when motors sre running
*/

if (doSecond){
    doSecond = false;                              // only do this once per second

  if (debug) digitalWrite(77, ! digitalRead(77));  // toggle the pin
  
  // ------ SOFTWARE CLOCK ------
  
  // The 1-second interrupts come from the RTCC so they should not loose sync
  // The date is updated from the RTCC at midnight
  // the time is in BCD format
  
  if (brakeTimer > 0){                           // motor brakes on?
    --brakeTimer;                                // long enough?
    if (brakeTimer == 0) BrakeOff();             // yes..turn them off
  }
  
  ++second ;                                     // increment the clock
  if ( second & 0x0f > 9){                       // digits 0 to 9 are ok
    second = second & 0xf0;                      // >9 so clear units nybble
    second = second + 0x10;                      // and increment tens nybble
  }
  if (second == 0x60){                           // minute mark?
    second = 0;
    
// ------ MINUTE INTERVAL ------
    
    doMinute = true;                             // yes..time to do the minute jobs
    if (minuteTimer > 0) --minuteTimer;          // process the job timers
    ++minute;
    if (minute & 0x0f > 9){
      minute = minute & 0xf0;
      minute = minute + 0x10;
    }
    if (minute == 0x60){
      minute = 0;
      
// ------ hour interval ------      
      
      ++hour;
      doHour = true;                            // time to do the hourly tasks
    if (hour & 0x0f > 9){ 
      hour = hour & 0xf0;
      hour = hour + 0x10;
    }
    if (hour == 0x24){
      hour = 0;
      doDay = true;                            // time to do the daily (midnight) jobs
    }
  }
 }
  
  Light();                                      // read the light levels every second
    
  // ------ STATISTICS ------

   switch (skyCondition){
      case CLOUDY:                               // another second of cloud
      dataCloud ++;
        break;
      
      case HAZY:                                 // another second of haze                        
        dataHaze ++;
        break;
        
      case SUNNY:                                // another second of sun
        dataSun ++;
        break;
    }

  // in verbose (tell-all) mode, print the direction and other data every second
  
  if(verbose){                                    // if in verbose mode
    if (vector > 0){                              // and a motor is running
      switch (vector){
        case 'E':
          Serial.print("E ");                     // print direction
          Serial.print(lightEast,DEC);            // east light intensity
          Serial.print(" ");
          Serial.print(lightWest,DEC);            // west light intensity
          Serial.print(" ");
          Serial.print(azp,DEC);                  // azimuth pulse count
          Serial.print(" ");
          Serial.print(azi,DEC);                  // azimuth motor current
          Serial.print(" ");
          Serial.println(pulseWidth,DEC);
          break;
        case 'W':
          Serial.print("W ");                     // print direction
          Serial.print(lightEast,DEC);
          Serial.print(" ");
          Serial.print(lightWest,DEC);
          Serial.print(" ");
          Serial.print(azp,DEC);                  // azimuth pulse count
          Serial.print(" ");
          Serial.print(azi,DEC);                  // azimuth motor current
          Serial.print(" ");
          Serial.println(pulseWidth,DEC);
          break;
        case 'U':
          Serial.print("U ");                     // print direction
          Serial.print(lightUp,DEC);
          Serial.print(" ");
          Serial.print(lightDown,DEC);
          Serial.print(" ");
          Serial.print(elp,DEC);                  // azimuth pulse count
          Serial.print(" ");
          Serial.print(eli,DEC);                  // azimuth motor current
          Serial.print(" ");
          Serial.println(pulseWidth,DEC);
          break;
        case 'D':
          Serial.print("D ");                     // print direction
          Serial.print(lightUp,DEC);
          Serial.print(" ");
          Serial.print(lightDown,DEC);
          Serial.print(" ");
          Serial.print(elp,DEC);                 // azimuth pulse count
          Serial.print(" ");
          Serial.print(eli,DEC);                 // azimuth motor current
          Serial.print(" ");
          Serial.println(pulseWidth,DEC);
          break;
        default:
          bug=BUG1;                              // software error trap - invalid direction
      }
    }
  }
}

/*-------------------------------- MINUTE TASKS ----------------------------------

      1- Read the air and PV temperatures every minute
      2- Update the daily minimum and maximum temperatures if required
      3- Track the sun. (This provides a potential of 1/4 degree resolution)
      
*/

if (doMinute){
    doMinute = false;                               // only do this once per minute

// ------ TRACK THE SUN ------

    Track();                                        // track the sun every minute
    
// ------ PARK THE ARRAY ------

if (skyCondition == DARK){;                         // if it has been dark for 2 hours...
    if ( ! PARKED){;
      if (minuteTimer == 0) Park();                 // park the array facing tomorrow's sunrise
    }
}
// ------ CALIBRATE THE PPH ------

if (pph == 0){;                                     // if pph has not been calibrated
  if (minuteTimer == 0){;                           // and we waited for 2 hours after sunrise
    if (! doCal){;                                  // and we are not counting yet
      if (skyCondition == SUNNY){                   // and it is now sunny
        calPulse = azp;                             // save the current position
        doCal = true;                               // wait for 1 hour
        minuteTimer = 60;
        }
      }
    else                                                 // we've been counting for an hour
      if (skyCondition == SUNNY) pph = (azp - calPulse); // still sunny so set the pph value
      doCal = false;                                     // done
    }
}
    
       
  // ------ READ THE TEMPERATURES ------
  
  byte celsius = 255;
  celsius = Temperature(pinPvTemp);                 // read the PV temperature sensor
  if (celsius < 255) pvTemp = celsius;              // range check (255= failure)
  
  celsius = Temperature(pinAirTemp);                // read the air temperature sensor
  if (celsius != 255){                              // range check (255= failure)
    airTemp = celsius;
    if (airMax == 255){                              // have the daily min & max been set?
      airMax = airMin = airTemp;                     // no...first reading of the day
      maxTime = minTime = (hour <<8 || minute);      // maxTime is an int & hour & minute are bytes
      }
    }
    if (airTemp > airMax){
      airMax = airTemp;                             // record the daily maximum air temperature
      maxTime = (hour <<8 || minute);               // maxTime is an int & hour & minute are bytes
    }
    if (airTemp < airMin){                          // record the daily minimum air temperature
      airMin = airTemp;                             // record the daily maximum air temperature
      minTime = (hour <<8 || minute);
     }

    doMinute = false;                                     // minute jobs are done
}

/*-------------------------------- HOURLY TASKS ----------------------------------

        1- manual mode reminder message
        2- alarm reminder message
        3- 
*/

if (doHour){
    doHour = false;                                      // only do this once per hour

  if(manual || alarm > 0){
    MessageHeader(100);                                  // print the message header
    
    if(manual) Serial.println("*** Manual mode");        // manual mode reminder
    if(alarm > 0){
      Serial.print("*** Alarm= ");                       // alarm reminder
      Serial.println(alarm);                             // print the alarm code
    }
  }
  
}

/*-------------------------------- DAILY TASKS ----------------------------------

        1- Update daily logs
        2- Update year to date logs
        3- Erase yesterday's daily data accumulators
        4- Get the date from the RTCC
        
    At midnight, the daily statistics are logged and added to the cumulative
    totals, then reset for the start of the next day. This includes the following items:
      - suny period in minutes (long)  (4 bytes)
      - hazy period in minutes (long)  (4 bytes)
      - cloudy period in minutes (long)  (4 bytes)
      - minimum temperateur (1 byte) and the time it first occurred (int) (2 bytes)
      - maximum temperateur (1 byte) and the time it first occurred (int) (2 bytes)
       
       6,606 bytes of the available 32,768 bytes are used for 1 year of data.
       The 24LC256 is rated for 1 million erase/write cycles so doing daily
       updates is not a problem (It would last about 2,000 years)
    
    doty       = index to the daily record (Day Of The Year)
                  (record 0= year to date data, records 1 to 366= daily data)
    recordSize = number of bytes in each record (18)

*/


if(doDay){                                               // jobs run at midnight
  doDay = false;                                         // only do this once per day

/*    +++ COMENTED OUT UNTIL THE LIBRARY CODE IS RE-WRITTEN +++

  byte doty;                                             // day of the year (used to index the data logs)
                                                         // calculate the day of the year
  byte index = 0
  int address = doty * 18                                // index to the start of the record to be written                                     

  if(data.Sun>60){                                       // range check
    int data= data.Sun / 60;                             // convert seconds to minutes
    if ((remainder % dataSun) > 29) data++;              // round up to the nearest minute
  else data = 0;                                         // ingore anything less than 1 minute per day
  }
  EEPROM.write(address,data);                             // write it to daily log
  temp = EEPROM.read (index);                             // add it to cumulative log
  EEPROM.write(index,(temp+data);
  index++; address++;

+++ expand the above to include the rest of the data +++

    // Temperature logs
  
                  // write it to daily log
                  // update annual temperature log
*/

// ------ RESET THE DAILY STATISTICS ------

  dataSun = dataHaze = dataCloud = 0;                    // reset the values
  airMin = airMax = 0xFF;                                // reset the temperatures to "not recorded"
}

//========================================================================================

/* ------ SNOWDUMP ------

      If mode = snowDump vibrate periodically to dump the snow off the array.
      The array is vertical so snow can slide off or not stick.
      This is called every 10 minutes and it vibrates for 10 seconds.
      
  +++ Add snow melt code here +++
*/

  if (snowDump){
      if (minute & 0x0F == 0){                                      // 10 minute intervals
        if (second < 10){
          if (digitalRead(pinVib)== 0) digitalWrite(pinVib,HIGH);   // start vibrator at 10 min, 0 sec
        else digitalWrite(pinVib,LOW);                              // turn it off after 10 seconds
      }
    }
  }

}                    // end of void loop()

//===================================== END OF THE DO FOREVER LOOP =============================



/*                                        <><><><><><><><><><><><><>
                                          <>                      <>
                                          <>       FUNCTIONS      <>
                                          <>                      <>
                                          <><><><><><><><><><><><><>
*/                                          

//------------------------------------- COMMAND LINE INTERPRETER -------------------------------

void Command (char rxChar){
  switch (rxChar){
    
    if(debug){
      case '!':
        doSecond = true;                   // simulate 1 second jobs
        break;
      case '@':
        second = 59;
        doSecond = true;                   // simulate minute jobs
        break;
      case '#':
        minute = 59;
        second=59;
        doSecond = true;                   // simulate hourly jobs
        break;
      case '$':
        doDay = true;                      // simulate daily (midnight) jobs
        break;
    }                                      // end of debug
        
    case 'a':
    case 'A':
      mode = AUTO;                         // Automatic mode
      manual = false;
      MessageHeader (2);
      Serial.println("    Automatic mode");
      break;
    
    case 'c':
      snowDump = true;                     // start periodic vibrations to clear the snow
      suspend = GoVertical();              // move the array to the vertical position
      MessageHeader (2);
      Serial.println("  * Snow dump: Moving to vertical position");
      break;
      
    case 'D':                              // move the elevation angle down (more vertical)
      manual = true;                       // enter manual mode
      MessageHeader (2);
      Serial.println("*** Manual mode: Down");
      Down();
      break;
      
    case 'E':                              // move the array east
      manual = true;
      MessageHeader (2);
      Serial.println("*** Manual mode: East");
      East();
      break;
      
    case 'F':                              // clear the fault flags
      fault = OK;
      alarm = OK;
      break;
      
    case 'l':                              // print the logs in .csv format
      break;
      
    case 'M':                              // enter manual mode
      manual = true;
      MessageHeader (2);
      Serial.println("*** Manual mode");
      Halt();                               // stop everything
      break;
      
    case 'N':                              // toggle one-eay tracking - east to west only (test mode)
      MessageHeader (2);
      if (oneWay){
        oneWay = false; 
        Serial.println("    Bi-directional tracking");
      }
      else{
        oneWay = true;
        Serial.println("  * East to west tracking only");
      }
      break;

    case 'O':                              // one axis tracking
      MessageHeader (2);
      if (singleAxis){
        singleAxis = false; 
        Serial.println("    Two axis tracking");
      }
      else{
        singleAxis = true;
        Serial.println("  * Single axis tracking only");
      }
      break;
      
    case 'P':                              // manually park the array
      manual = true;
      MessageHeader (2);
      Serial.println("*** Manual mode: Parking");
      suspend = Park();
      break;
      
    case 'Q':                              // query the status of everything
    case 'q':
      CmdQuery();
      break;
      
    case 'R':                              // restart - same as power on but without clwaring the data
      break;

    case  's':                             // enter snow mode (tracking with the array vertical)
      MessageHeader (2);
      Serial.print("    Snow mode: Array going vertical");
      snowMode = true;
      GoVertical();
      break;
      
    case 'S':                              // stop the array
      manual = true;
      Halt();
      MessageHeader (2);
      Serial.println("*** Manual mode: Array is stopped");
      break;
      
    case 'T':                              // test the hardware
      manual = true;
      break;
      
    case 'U':                              // move ther array up (flatter angle)
      manual = true;
      MessageHeader (2);
      Serial.println("*** Manual mode: Up");
      Up();
      break;
      
    case 'V':
      if (verbose){                       // toggle verbose mode on or off
        verbose = false; 
        Serial.println("Verbose mode off");
      }
      else{
        verbose = true;
        Serial.println("Verbose mode on");
      }
      break;
      
    case 'v':                              // report software version
      MessageHeader(001);
      Serial.print("    SunSeeker (c) 2012 by G.Pearen Ver: ");
      Serial.println(ver,HEX);             // print the date without any separators (example 20120318) 
      break;
      
    case 'W':                              // move the array west
      manual = true;
      MessageHeader (2);
      Serial.println("*** Manual mode: West");
      West();
      break;
      
    case 'X':                              // stow the array for high cross-winds
      manual = true;
      suspend = Stow();
      MessageHeader (2);
      Serial.println("*** Manual mode: Stowing");
      break;
      
    case 'Y':                              // return the array to position before stow command
      manual = false;
      suspend = UnStow();
      MessageHeader (2);
      Serial.println("    Tracking restored");
      break;
      
    case 'Z':                              // simulate a power-on reset
      break;
      
    default:                               // print a question mark
      Serial.println("?");
  }                                        // end of switch statement
  rxChar = 0;                              // command was processed so erase it (0= ASCII nul)
}                                          // end of Command function


//------------------------------- MESSAGE HEADER ---------------------------

void MessageHeader(byte message){           // print the message number
  Serial.print(message,DEC);
  Serial.print("  20");                     // print the date and time
  Serial.print(year,HEX);
  Serial.print("-");
  Serial.print(month,HEX);
  Serial.print("-");
  Serial.print(mday,HEX);
  Serial.print(" ");
  Serial.print(hour,HEX);
  Serial.print(":");
  Serial.print(minute,HEX);
  Serial.print(":");
  Serial.println(second,HEX);
}

//------------------------- Query command - report all data ---------------------------

void CmdQuery(){
  byte temp;
  MessageHeader(100);
  Serial.print("    ");
  Serial.print(lightEast,DEC);       // light intensity read by the east sensor
  Serial.print(" ");
  Serial.print(lightWest,DEC);       // light intensity read by the west sensor
  Serial.print(" ");
  Serial.print(lightDown,DEC);       // light intensity read by the down sensor
  Serial.print(" ");
  Serial.print(lightUp,DEC);         // light intensity read by the up sensor
  Serial.print(" ");
  Serial.print(lightSun,DEC);        // light intensity read by the sun sensor
  Serial.print(" ");
  Serial.print(lightSnow,DEC);       // light intensity read by the snow sensor
  Serial.print(" ");
  Serial.print(azp,DEC);             // azimuth pulse ount
  Serial.print(" ");
  Serial.print(elp,DEC);             // elevation pulse count
  Serial.print(" ");
  Serial.println(pph,DEC);           // PPH calibration data
  Serial.print("    ");
  PrintTemp(airTemp);                // current air temperature
  Serial.print(" ");
  
  PrintTemp(airMin);                 // today's minimum temperature
  Serial.print("@");
  Serial.print(highByte(minTime),HEX);
  Serial.print(":");
  Serial.print(lowByte(minTime),HEX);
  Serial.print(" ");
  
  PrintTemp(airMax);                // today's maximum temperature
  Serial.print("@");
  Serial.print(highByte(maxTime),HEX);
  Serial.print(":");
  Serial.print(lowByte(maxTime),HEX);
  Serial.print(" ");
  
  PrintTemp(pvTemp);                // PV temperature
  Serial.print(" ");
  Serial.print(azi,DEC);             // azimuth motor current
  Serial.print(" ");
  Serial.print(eli,DEC);             // elevation motor current
  Serial.print(" ");

  PrintSeconds (dataSun);            // print the amount of sun today in hh:mm
  PrintSeconds (dataHaze);           // print the amount of sun today in hh:mm
  PrintSeconds (dataCloud);          // print the amount of sun today in hh:mm

  Serial.print("\n    ");            // indent next line
  switch(skyCondition){
      case DARK:
        Serial.println("Too dark to track");
        break;
      case DAWN:
        Serial.println("Dawn");
        break;
      case CLOUDY:
        Serial.println("Cloudy");
        break;
      case HAZY:
        Serial.println("Hazy - tracking");
        break;
      case SUNNY:
        Serial.println("Sunny - tracking");
        break;
      case DUSK:
        Serial.println("Dusk");
        break;
      default:
        bug = BUG2;                          // invalid sky condition
  }

  Serial.print("    ");                      // indent next line
  
  switch(mode){
      case 0:
        Serial.print("Waiting for data");    // no sensor head attached
        break;
      case 'P':
        Serial.print("Parked ");
        break;
      case 'T':
        Serial.print("Tracking ");
        break;
      case 'X':
        Serial.print("Stowed ");
        break;
      case 'S':
        Serial.print("Snow dump ");
        break;
      Default:
        bug = BUG4;
  }
  Serial.print("\n");                                            // more than one of these could be printed
  if (manual)     Serial.println ("    Manual mode");
  if (oneWay)     Serial.println ("    One-way tracking mode");
  if (singleAxis) Serial.println ("    Single axis tracking mode");
  if (snowMode)   Serial.println ("    Snow mode");
  if (vertical)   Serial.println ("    Array is vertical");
  Serial.print("\n");
}                                                    // end of Query command

// ------ FORMAT & PRINT TEMPERATURE ------

void PrintTemp(byte temp){
  if (temp == 255){
    Serial.print("___");                    // if invalid print "___"
    return;
  }
  if (! (temp & 0x80)) Serial.print("+");   // positive temperature
  else Serial.print("-");

  Serial.print(temp & 0x7F,DEC);            // negative temperature
}

//------ FORMAT AND PRINT SECONDS AS HOURS:MINUTES ------

void PrintSeconds(int time){
  Serial.print((time/3600),DEC);            // print hours
  Serial.print(":");
  Serial.print((time%3600/60),DEC);         // print minutes
  Serial.print(" ");                        // forget the seconds
}

/*------------------------------------------- SET THE CLOCK ---------------------------------------

                          +++ Complete this when the RTCC is implemented +++

      Date format:  year, month,  day,    wkday
      Time format:  hour, minute, second, reserved (0)
      
      BCD format:   Each field is 8 bits (4 bits per digit), (0x16 = day 16)

The following are defined as unsigned characters in rtcc.h and use BCD coding
  hour, minute, second, rsvd
  year, mon, mday, wday
*/

/*------------------------------------------- FAULTS AND WARNINGS ---------------------------------------

+++ Change fault recording and printing method +++

*/

void MotorTrouble(){                       // motor fault detected
  alarm = fault;                           // save the alarm condition
  fault = 0;
  MessageHeader(alarm);                    // print the message header
  
    if(azfail){
      Serial.print("*** Azimuth motor trouble while moving ");
      if(azWest) Serial.println("west");
      else Serial.println("east");
      Serial.print("    ");
      if(azNoPulse) Serial.print("No pulse detected. ");
      if(azHbridge) Serial.print("H-Bridge fault. ");
      if(azLimitSw) Serial.print("Limit switch operated. ");
      if(azCurrentHi) Serial.print("Motor is overloaded. Current= ");
      if(azCurrentLow) Serial.print("Motor is unloaded. Current= ");
      if(azCurrentLow || azCurrentHi) Serial.print(azi,DEC);
      Serial.println("\n");
    }

    if(elfail){
      Serial.print("*** Elevation motor trouble while moving ");
      if(elUp) Serial.println("up");
      else Serial.println("down");
      Serial.print("    ");
      if(elNoPulse) Serial.print("No pulse detected. ");
      if(elHbridge) Serial.print("H-Bridge fault. ");
      if(elLimitSw) Serial.print("Limit switch operated. ");
      if(elCurrentHi) Serial.print("Motor is overloaded. Current= ");
      if(elCurrentLow) Serial.print("Motor is unloaded. Current= ");
      if(elCurrentLow || elCurrentHi) Serial.print(eli,DEC);
      Serial.println("\n");
   }
}

/* ---------------------- Analyze a motor alarm and attempt to recover from it -------------------------

               +++ fill in the details later +++
*/

void ProcessFault(){
  
  
}


/*------------------------------------------------ PARK ----------------------------------------------

    Park the array after dark facing tomorrow's sunrise
      - go east until the counter = 0 which is the sunrise position
      - go down until the array is vertical
      - If the vertical position has ben calibrated, it should stop just before the limit switch operates
*/

byte Park() {
    if (azp > AZPARK){                              // azimuth parking
      if (! digitalRead(pinAzEnable)) East;         // start east movement if not already done
      return 'P';                                   // let it continue
    }
    Halt();                                         // stop the azimuth motor when done
    
    if (elp > ELPARK) {                             // elevation parking
      if (! digitalRead(pinElEnable)) Down;         // start down movement if not already done
      return 'P';                                   // let it run
    }
    Halt();                                         // stop the motor
    mode= PARKED;                                   // show that the array is parked
    return OK;                                      // and the job is done
}

//-------------------------------------------- VERTICAL ------------------------------------------

byte GoVertical(){
    if (elp >= ELPARK) {                            // stop before limit switch operation
      if (not digitalRead(pinElEnable)) Down;       // if the H-bridge is not already enabled, do it
      return 'V';                                   // suspend until the array is vertical
      }
    Halt();                                         // array is now vertical
    vertical = true;
    return OK;
}

/*------------------------------------------------- STOW ------------------------------------------------

      stow the array flat during high winds
      This disables all tracking by putting SunSeeker into manual mode 
*/

byte Stow(){
    elpStow = elp;                                    // save the current elevation count (global)
    if (elp <= ELSTOW) {                              // stop before limit switch operation
      if (! digitalRead(pinAzEnable)) Up;             // start the motor if required
    return 'X';                                       // let it run
    }
    Halt();                                           // stop the motor
    mode= STOWED;                                     // show the array stowed
    manual = true;                                    // disable automatic operation
    return OK;                                        // end the job
}

/*------------------------------------------- UN-STOW THE ARRAY ------------------------------------------

        Restore the array position after being stowed for protection from high winds
*/

byte UnStow() { 
    if (elp > elpStow) {                              // return to previous position
      if (! digitalRead(pinAzEnable)) Down;           // start the motor if required
    return 'Y';                                       // let it run
    }
    Halt();                                           // stop the motor
    mode= AUTO;                                       // switch to automatic mode
    manual = false;
    return OK;                                        // end the job
}

 /*-------------------------------------------- SCAN FOR THE SUN ---------------------------------------
 
         - Go west in manual mode until the west light is stronger than east light
         - Then switch to automatic mode
 
 */
 
 byte Scan (){
   if (! digitalRead(pinAzEnable)){
     West();                                            // start west tracking
     manual = true;                                     // enter manual mode
     return 'S';                                        // suspend this job
   }
     if (lightWest <= lightEast) return 'S';            // continue scanning
     manual = false;                                    // restore automatic mode
     return OK;                                         // end this job
       
 }
 
  /*--------------------------------------------- TRACKING -----------------------------------------------
  
          - Compare the light readings
          - Move the array if requred
          - Skip azimuth tracking if it is failed
          - skip elevation tracking if it is failed
          - skip elevation tracking if in snow mode (snowMode = true)
          - skip elevation tracking if there is snow on the array (snowDump = true)
          - skip elevation tracking if low light and temperature below freezing (vertical = true)
   */
  
void Track() {
  if (! azfail){
     if (lightWest > lightEast) {
       if (! digitalRead(pinAzEnable)) West();            // start west tracking
     }
     
    if (lightEast > lightWest) {
      if (! digitalRead(pinAzEnable)) East();             // start tracking east
    }
  }
  
  if (! (elfail || snowMode || snowDump || vertical)){     // abort if elevation is turned off
    if (lightUp > lightDown) {
      if (! digitalRead(pinElEnable)) Up();                // start tracking up
    }

    if (lightDown > lightUp) {
      if (! digitalRead(pinElEnable)) Down();             // start tracking down
    }
  }
}

/*----------------------------------------- READ THE LIGHT INTENSITY SENSORS -------------------------------

           - Read all the light sensors, use the upper 8 bits only
           - take 4 teadings and average them
           - 'light.xxx' are global variables, 'temp' and "i" are local
       EXIT:
           - light levels are set
           - sky.condition is set
*/

void Light (){
  int temp=0;
  for(int i=0; i<5; i++) temp = temp + (analogRead(pinEastSensor) >> 2);
  lightEast = temp >> 2;
    
  temp=0;
  for(int i=0; i<5; i++) temp = temp + (analogRead(pinWestSensor) >> 2);
  lightWest = temp >> 2;

  temp=0;
  for(int i=0; i<5; i++) temp = temp + (analogRead(pinUpSensor) >> 2);
  lightUp = temp >> 2;

  temp=0;
  for(int i=0; i<5; i++) temp = temp + (analogRead(pinDownSensor) >> 2);
  lightDown = temp >> 2;

  temp=0;
  for(int i=0; i<5; i++) temp = temp + (analogRead(pinSunSensor) >> 2);
  lightSun = temp >> 2;
  
  switch(skyCondition){
    case DARK:
      if (lightSun < cloud) skyCondition = DAWN;        // sunrise
      break;
  
    case DUSK:                                          // if the sun has set
      if (lightSun <= dawn-1){                          // and if it is getting darker (use a little hysterises)
        skyCondition = DARK;                            // night...
        minuteTimer = 120;                              // wait 2 hours then park the array
      break;
      }
       
    default:
      if (lightSun < dawn)      skyCondition = DUSK;     // sunset
      else if (lightSun < haze) skyCondition = CLOUDY;   // cloudy
      else if (lightSun < sun)  skyCondition = HAZY;     // hazy
      else{
        skyCondition = SUNNY;                            // sunny
        if (pph == 0) minuteTimer = 120;                 // wait 2 hours (early morning light refraction)
    }
  }
      
  temp=0;
  for(int i=0; i<5; i++) temp = temp + (analogRead(pinSnowSensor) >> 2);
  lightSnow = temp >> 2;
}

/*----------------------------------------- READ THE TEMPERATURE SENSORS -------------------------------

      ENTER: pin number in "pin" = pinAirTemp (air temperature) or pinPvTemp (PV temperature)
      EXIT:  temperature in "celsius". Signed byte format
             FF= sensor not detected
*/

byte Temperature (byte pin){
  byte present;
  byte lsbyte;
  byte msbyte;
  byte celsius = 0xFF;                          // FF= error flag

/*       +++ deleted temporarily +++

  OneWire.OneWire(pin);                         // identify the pin used for data
  present = OneWire.reset();                    // reset the sensor
  OneWire.skip;                                 // skip reading the ROM
  OneWire.write(0x44);                          // start the sensor
  delay(750);                                   // it takes 750ms maximum to get the temperture
    
  present = OneWire.reset();
  if (! present) return celsius;                // celsius= FF if sensor is not responding
  
  OneWire.skip;                                 // skip reading the ROM
  OneWire.write(0xBE);                          // issue a read command
  lsbyte = OneWire.read();                      // read the temperature
  msbyte = OneWire.read();                      // read the sign bit
*/

  /*  Format the data to temperature
      - msbyte is all 1's for a negative temperature.
      - lsbyte bit-0 is the .5 degree bit so shift it R one place to get whole degrees
      - Set the sign bit (bit 7) of celsius if the temperature was negative
  */

  celsius = lsbyte >> 1;
  if (msbyte > 0) celsius = celsius || 0x80;    // set the sign bit for negative nemperatures
  return celsius;
}

/*----------------------------------------- PV array motor drive commands -------------------------------

    Halt:       Stop all motion and exit with all H-bridge controls inactive
    
    East:        move the array east
    West:        move the array west
    Up:          move the array up
    Down:        move the array down
       
    WatchMotor:  - Watch for H-Bridge errors
                 - Count the pulses
                 - Check for a pulse indicating movement
                 - Check for no current draw (motor open or broken mechanical linkage)
                 - Check for excess curent (mechanical failure)
                 - Convert the data from the H-bridge FB pin milliamps
  
                  EXIT:    azp      = azimuth pulse count updated (global)
                           elp      = elevation pulse count updated (global)
                           eli      = elevation motor current (global)
                           azi      = azimuth motor current (global)
                           fault = hardware trouble code set (global)
*/

byte WatchMotor(){
  switch (vector){
    case 'E':                               // azimuth
    case 'W':
      pulseWidth = pulseIn(pinAP,LOW);                         // watch for a pulse
      if (pulseWidth == 0){
        azi=(analogRead(pinAzFeedback)*14);                    // no pulse - read the current
        Halt();                                                // stop the motor
        azfail = true;
        azNoPulse = true;
        return azfail;
      }
      else {
        if (vector == 'W') azp ++; else azp--;                 // count the pulse - west= increment, east= decriment
        if (azp < 0) azp = 0;                                  // don't let it go negative
        azi = (analogRead(pinAzFeedback)*14);                  // read the motor current in ma
        if (azi < IMin){
          Halt();
          azfail=true;
          azCurrentLow = true;
          return azfail;                                       // current too low
          }
        if (azi > IMax){
          Halt();
          azfail = true;
          azCurrentHi = true;                                  // current too high
          return azfail;
          }
        return OK;
      }
    break;
      
    case 'U':                              // elevation
    case 'D':
      pulseWidth = pulseIn(pinEP,LOW);                         // watch for a pulse
        if (pulseWidth == 0){
        eli=(analogRead(pinElFeedback)*14);                    // no pulse - read the current
        Halt();                                                // stop the motor
        elfail = true;
        elNoPulse = true;
        return elfail;
      }
      else {
        if (vector == 'U') elp ++; else elp--;                 // count the pulse - west= increment, east= decriment
        if (elp < 0) elp = 0;                                  // don't let it go negative
        eli = (analogRead(pinElFeedback)*14);                  // read the motor current in ma
        if (eli < IMin){
          Halt();
          elfail=true;
          elCurrentLow = true;
          return elfail;                                       // current too low
        }
        if (eli > IMax){
          Halt();
          elfail = true;
          elCurrentHi = true;                                  // current too high
          return elfail;
          }
        return OK;
      }
      break;
        
      default:
        return BUG1;                                           // invalid condition
  }
}


/*    ------ note ------

     The PIC is 5 volt tolerant on many input pins but you can't use
     pull-up resistors to 5v to get 5v outputs. The PIC will clamp the
     outputs to 3.3v. Instead of using a level shifter, you can use 
     pull-ups and switch the pins to high impedance (input)
     to send a 5v logic high to a peripheral. That is what is done here
*/

void Halt(){
  pinMode(pinHBridgeIn1,OUTPUT);
  pinMode(pinHBridgeIn2,OUTPUT);
  pinMode(pinAzEnable,OUTPUT);
  pinMode(pinElEnable,OUTPUT);
  digitalWrite(pinHBridgeIn1,LOW);        // enable both LOW side drivers
  digitalWrite(pinHBridgeIn2,LOW);        // this stops & brakes the motor
  digitalWrite(pinVib,LOW);               // make sure the vibrator is off too
  vector = 0;                             // stop watching the motor
  brakeTimer = 2;                         // brake the motors for 2 seconds
}

void BrakeOff(){                          // take the brakes off the motors
  digitalWrite(pinAzEnable,LOW);          // disable the azimuth H-bridge
  digitalWrite(pinElEnable, LOW);         // disable the elevation H-bridge
  }
  
void West(){
  pinMode(pinHBridgeIn1,INPUT);           // let the 5v pull up resistors do their thing
  pinMode(pinAzEnable,INPUT);
  azWest = true;
  vector='W';
  }
  
void East(){
  pinMode(pinHBridgeIn2,INPUT);
  pinMode(pinAzEnable,INPUT);
  azWest = false;
  vector = 'E';
  }
  
void Up(){
  pinMode(pinHBridgeIn1,INPUT);
  pinMode(pinElEnable,INPUT);
  elUp = true;
  vector = 'U';
  }
  
void Down(){
  pinMode(pinHBridgeIn2,INPUT);
  pinMode(pinElEnable,INPUT);
  elUp = false;
  vector = 'D';
  }
  
/*===================================================================

     +++ Copied from time_date.c & extensively modified +++
          (It still doesn't work)

*********************************************************************/

//#include <plib.h>                // Master header file for all peripheral library includes

// Configuration Bit settings

//#pragma config FPLLMUL = MUL_20, FPLLIDIV = DIV_2, FPLLODIV = DIV_1, FWDTEN = OFF
//#pragma config POSCMOD = HS, FNOSC = PRIPLL, FPBDIV = DIV_1

//  SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);


/*********************************************************************
       Exit:  true if everything went on ok, false if it failed

	 Configure the device for maximum performance but do not change the PBDIV
	 Given the options, this function will change the flash wait states, RAM
	 wait state and enable prefetch cache but will not change the PBDIV.
	 The PBDIV value is already set via the pragma FPBDIV option above.
	 when using the RtccSetTimeDate() function, the write operation is enabled when needed

	 time is MSb: hour, min, sec, rsvd      (the rsvd field must be 0)
         date is MSb: year, mon, mday, wday.
	 
      Examples:
	RtccGetTimeDate(&tm1, &dt1);      // read the time and date in a single operation
        tm1.l=RtccGetTime();              // read the time only
        dt1.l=RtccGetDate();              // read the date only

	The date & time can also be set this way
	{
	tm.l=0;
	tm.sec=0x30;
	tm.min=0x07;
	tm.hour=0x10;

	dt.wday=2;
	dt.mday=0x16;
	dt.mon=0x01;
	dt.year=0x07;

	RtccSetTimeDate(tm.l, dt.l);
	}
*******************************************************************************************/

boolean Rtcc();

boolean Rtcc(void){

  boolean  CheckRtccRunning(int secWait);                 //  local function prototype
  
  rtccTime tm, tm1;                                      // time structure
  rtccDate dt, dt1;                                      // date structure

  RtccInit();			                          // reset the RTCC
  while(RtccGetClkStat() != RTCC_CLK_ON);	          // wait until the SOSC is running so RTCC has its clock source
					                  // could wait here at most 32ms (need an error timeout)
//  RtccSetTimeDate(tm, dt);                              // set the date and time to match the sfw clock                
  RtccGetTimeDate(&tm1, &dt1);                            // read the date & time from the RTCC (into scratch variabled)
  if(tm.hour!=tm1.hour || tm.min!=tm1.min) return false;  // does the time match what we just set
  if(dt.l!=dt1.l) return false;                           // check the date too
  return true;                                            // ok
}

// -----------------------------------------------------------------------------------------------
//     Output:          true if test succeeded, false if it failed
//     Overview:        The function checks that the RTCC has the clock enabled and counts the time.
// -------------------------------------------------------------------------------------------------

boolean CheckRtccRunning(int secWait){
  rtccTime t0, t1;
  
//  unsigned int  tStart;

  delay (secWait);
//  tStart = ReadCoreTimer();
  t0.l = RtccGetTime();
  if (t1.sec == t0.sec) return false;          // failed...seconds not incrementing
  return true;                                 // ok
  }
  
// now that we know the RTCC clock is up and running, it's easier to start from fresh:
  
//  RtccOpen(tm.l, dt.l, 0);                    // set time, date and calibration in a single operation
//  if(!CheckRtccRunning(3))  return false;    	// check that the RTCC is running


//     other RTCC operations

//     adjust the RTCC timing

//  RtccSetCalibration(200);	        // value to calibrate with at each minute
	                                // enabling the RTCC output pin
//  RtccSelectPulseOutput(1);		// select the seconds clock pulse as the function of the RTCC output pin
//  RtccSelectPulseOutput(0);		// select the alarm pulse as the function of the RTCC output pin
//  RtccOutputEnable(1);			// enable the Output pin of the RTCC
//  return true;
//}

